#! /opt/local/bin/perl
##
## Converts Open-shop Scheduling Problem into ASP facts
##
## See Open-shop scheduling web page in OR-Library
##     http://people.brunel.ac.uk/~mastjjb/jeb/orlib/openshopinfo.html
##
## This program is a part of Sugar (a SAT-based Constraint Solver)
## software package.
##     http://bach.istc.kobe-u.ac.jp/sugar/
## (C) Naoyuki Tamura
##
use Getopt::Std;
use strict;
$| = 1;

use vars qw($opt_h);

&getopts("h:");

if ($opt_h) {
    print "Usage: $0 file.oss >file.lp\n";
    exit(1);
}

my $n;
my @pt = ();

&read_oss();
&write_asp();

exit 0;

sub read_oss {
    chomp($_ = <>);
    @_ = split;
    @pt = ();
    if (@_ == 2 && $_[0] == $_[1]) {
	$n = $_[0];
	while (<>) {
	    tr/\r\n//d;
	    @_ = split;
	    my $m = shift(@_);
	    if ($m == 0) {
		last;
	    }
	    my @s = ();
	    while (@_) {
		my $i = shift(@_);
		$s[$i] = shift(@_);
	    }
	    if ($m != $n || @s != $n) {
		@pt = ();
		last;
	    }
	    push(@pt, \@s);
	}
	if (@pt != $n) {
	    @pt = ();
	}
    }
    if (@pt == 0) {
	print "% ERROR : read format error\n";
    }
}

sub write_asp {
    print "% OSS\n";
    print "% $n $n\n";
    foreach (@pt) {
	print "% ", join(" ", @{$_}), "\n";
    }
    my $lb = &lower_bound();
    my $ub = &upper_bound();
    print "\n";
    print "#const lb = $lb.\n";
    print "#const ub = $ub.\n";
    print "job($n).\n";
    print "machine($n).\n";
    for (my $i=0; $i<@pt; $i++) {
	my @x = @{$pt[$i]};
	for (my $j=0; $j<@x; $j++) {
	    print "p(", join(",", ($i,$j,$x[$j])), ").\n";
	}
    }
    print "\n% END\n";
}

sub lower_bound {
    my $x;
    foreach my $i (0 .. $n - 1) {
	my $s = 0;
	foreach my $j (0 .. $n - 1) {
	    $s += $pt[$i][$j];
	}
	if (! $x || $x < $s) {
	    $x = $s;
	}
    }
    foreach my $j (0 .. $n - 1) {
	my $s = 0;
	foreach my $i (0 .. $n - 1) {
	    $s += $pt[$i][$j];
	}
	if (! $x || $x < $s) {
	    $x = $s;
	}
    }
    return $x;
}

sub upper_bound {
    my $s = 0;
    foreach my $k (0 .. $n - 1) {
	my $x = 0;
	foreach my $i (0 .. $n - 1) {
	    my $j = ($i + $k) % $n;
	    if (! $x || $x < $pt[$i][$j]) {
		$x = $pt[$i][$j];
	    }
	}
	$s += $x;
    }
    return $s;
}
